# script1.py

import json
import re
from common import (
    compare_headings,
    preprocess_data,
    transform_json,
    post_process,
    process_nested_data,
)

# 对于每个 target_value 元素，如果有完美匹配 json_data 中的键，那就加入这个完美匹配的键名，否则，把全部模糊匹配到的键名都加入
def find_keys_by_value(target_value, json_data):
    matched_keys = [k for k, v in json_data.items() if v == target_value]  # 首先检查 JSON 中的每个键值对，如果值完全等于目标值，则将这些键收集起来。
    if not matched_keys:
        matched_keys = [
            k
            for k, v in json_data.items()
            if isinstance(v, str) and v.startswith(target_value)
        ]  # 如果没有找到完全匹配的键，它会检查字符串类型的值是否以目标值开头，并收集这些键。
    return matched_keys  # 例如：[3.1,3.1.1,3.1.2,3.2...]

# 定义查找以特定前缀开始的键的函数，例如：若 match_keys 中有 3.1，那么以 3.1 为前缀的键都会被找出来，如 3.1.1、3.1.2...
def find_keys_with_prefix(key_prefix, json_data):
    subheadings = [k for k in json_data if k.startswith(key_prefix)]
    return subheadings

# 从完整的 JSON 文件中读取所需数据，例如：投标、评标
def extract_json(data, target_values):
    results = {}
    for target_value in target_values:
        matched_keys = find_keys_by_value(target_value, data)
        for key in matched_keys:
            key_and_subheadings = find_keys_with_prefix(key, data)
            for subkey in key_and_subheadings:
                if "." in subkey:
                    parent_key = subkey.rsplit(".", 1)[0]
                    top_level_key = parent_key.split(".")[0] + "."
                    # 特别处理定标相关的顶级键，确保不会重复添加其他键
                    if top_level_key not in results:
                        results[top_level_key] = target_value
                    # 添加或更新父级键
                    if parent_key not in results:
                        if parent_key in data:
                            results[parent_key] = data[parent_key]
                # 添加当前键
                results[subkey] = data[subkey]
    return results

# 读取 JSON 数据，提取内容，转换结构，并返回结果
def extract_from_notice(clause_path, type):
    if type == 1:
        target_values = ["投标", "投标文件", "响应文件"]
    elif type == 2:
        target_values = ["开标", "评标", "定标", "磋商程序", "中标"]
    elif type == 3:
        target_values = ["重新招标、不再招标和终止招标", "重新招标", "不再招标", "终止招标"]
    elif type == 4:
        target_values = ["评标"]  # 测试
    else:
        raise ValueError(
            "Invalid type specified. Use 1 for '投标文件, 投标' or 2 for '开标, 评标, 定标' or 3 for '重新招标'"
        )
    with open(clause_path, "r", encoding="utf-8") as file:
        data = json.load(file)
        extracted_data = extract_json(data, target_values)  # 读取 JSON
        # print(json.dumps(extracted_data, ensure_ascii=False, indent=4))
        sorted_data = preprocess_data(extracted_data)  # 对输入的字典 data 的键进行预处理和排序
        transformed_data = transform_json(sorted_data)
        # print(json.dumps(transformed_data, ensure_ascii=False, indent=4))
        final_result = process_nested_data(transformed_data)
        return final_result

if __name__ == "__main__":
    # 测试代码
    # file_path = 'C:\\Users\\Administrator\\Desktop\\fsdownload\\3bffaa84-2434-4bd0-a8ee-5c234ccd7fa0\\clause1.json'
    file_path = "C:\\Users\\Administrator\\Desktop\\招标文件\\special_output\\clause1.json"
    try:
        res = extract_from_notice(file_path, 2)  # 可以改变此处的 type 参数测试不同的场景
        res2 = json.dumps(res, ensure_ascii=False, indent=4)
        print(res2)
    except ValueError as e:
        print(e)
